# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: matsim/pb/Ids.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13matsim/pb/Ids.proto\x12\tmatsim.pb\"E\n\x07ProtoId\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05index\x18\x02 \x01(\x05\x12\x1f\n\x04type\x18\x03 \x01(\x0e\x32\x11.matsim.pb.IdType*\x9b\x01\n\x06IdType\x12\x0e\n\nID_UNKNOWN\x10\x00\x12\x10\n\x0cID_DEPARTURE\x10\x01\x12\x14\n\x10ID_TRANSIT_ROUTE\x10\x02\x12\x13\n\x0fID_TRANSIT_LINE\x10\x03\x12\x0e\n\nID_VEHICLE\x10\x04\x12\x0b\n\x07ID_LINK\x10\x05\x12\r\n\tID_PERSON\x10\x06\x12\x18\n\x14ID_ACTIVITY_FACILITY\x10\x07\x42\x1c\n\x18org.matsim.core.utils.pbP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'matsim.pb.Ids_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030org.matsim.core.utils.pbP\001'
  _globals['_IDTYPE']._serialized_start=106
  _globals['_IDTYPE']._serialized_end=261
  _globals['_PROTOID']._serialized_start=34
  _globals['_PROTOID']._serialized_end=103
# @@protoc_insertion_point(module_scope)
