# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: matsim/pb/Wireformat.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1amatsim/pb/Wireformat.proto\x12\tmatsim.pb\"L\n\x0cPBFileHeader\x12\x0f\n\x07version\x18\x01 \x01(\r\x12+\n\x0b\x63ontentType\x18\x02 \x01(\x0e\x32\x16.matsim.pb.ContentType*&\n\x0b\x43ontentType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\n\n\x06\x45VENTS\x10\x01\x42\x1c\n\x18org.matsim.core.utils.pbP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'matsim.pb.Wireformat_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030org.matsim.core.utils.pbP\001'
  _globals['_CONTENTTYPE']._serialized_start=119
  _globals['_CONTENTTYPE']._serialized_end=157
  _globals['_PBFILEHEADER']._serialized_start=41
  _globals['_PBFILEHEADER']._serialized_end=117
# @@protoc_insertion_point(module_scope)
