# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: matsim/pb/Events.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from matsim.pb import Ids_pb2 as matsim_dot_pb_dot_Ids__pb2
from matsim.pb import Model_pb2 as matsim_dot_pb_dot_Model__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16matsim/pb/Events.proto\x12\tmatsim.pb\x1a\x13matsim/pb/Ids.proto\x1a\x15matsim/pb/Model.proto\".\n\nEventBatch\x12 \n\x06\x65vents\x18\x01 \x03(\x0b\x32\x10.matsim.pb.Event\"\xaa\x07\n\x05\x45vent\x12\x0c\n\x04time\x18\x01 \x01(\x01\x12%\n\x06\x63oords\x18\x02 \x01(\x0b\x32\x15.matsim.pb.Coordinate\x12*\n\x07generic\x18\x03 \x01(\x0b\x32\x17.matsim.pb.GenericEventH\x00\x12\x32\n\x0b\x61\x63tivityEnd\x18\x04 \x01(\x0b\x32\x1b.matsim.pb.ActivityEndEventH\x00\x12\x36\n\ractivityStart\x18\x05 \x01(\x0b\x32\x1d.matsim.pb.ActivityStartEventH\x00\x12.\n\tlinkEnter\x18\x06 \x01(\x0b\x32\x19.matsim.pb.LinkEnterEventH\x00\x12.\n\tlinkLeave\x18\x07 \x01(\x0b\x32\x19.matsim.pb.LinkLeaveEventH\x00\x12\x38\n\x0fpersonalArrival\x18\x08 \x01(\x0b\x32\x1d.matsim.pb.PersonArrivalEventH\x00\x12:\n\x0fpersonDeparture\x18\t \x01(\x0b\x32\x1f.matsim.pb.PersonDepartureEventH\x00\x12\x42\n\x13personEntersVehicle\x18\n \x01(\x0b\x32#.matsim.pb.PersonEntersVehicleEventH\x00\x12\x42\n\x13personLeavesVehicle\x18\x0b \x01(\x0b\x32#.matsim.pb.PersonLeavesVehicleEventH\x00\x12\x32\n\x0bpersonMoney\x18\x0c \x01(\x0b\x32\x1b.matsim.pb.PersonMoneyEventH\x00\x12\x32\n\x0bpersonStuck\x18\r \x01(\x0b\x32\x1b.matsim.pb.PersonStuckEventH\x00\x12\x42\n\x13transitDriverStarts\x18\x0e \x01(\x0b\x32#.matsim.pb.TransitDriverStartsEventH\x00\x12\x36\n\rvehicleAborts\x18\x0f \x01(\x0b\x32\x1d.matsim.pb.VehicleAbortsEventH\x00\x12\x44\n\x14vehicleEntersTraffic\x18\x10 \x01(\x0b\x32$.matsim.pb.VehicleEntersTrafficEventH\x00\x12\x44\n\x14vehicleLeavesTraffic\x18\x11 \x01(\x0b\x32$.matsim.pb.VehicleLeavesTrafficEventH\x00\x42\x06\n\x04type\"}\n\x0cGenericEvent\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x31\n\x05\x61ttrs\x18\x02 \x03(\x0b\x32\".matsim.pb.GenericEvent.AttrsEntry\x1a,\n\nAttrsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x95\x01\n\x10\x41\x63tivityEndEvent\x12\"\n\x06linkId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12&\n\nfacilityId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12$\n\x08personId\x18\x03 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x0f\n\x07\x61\x63ttype\x18\x04 \x01(\t\"\x97\x01\n\x12\x41\x63tivityStartEvent\x12\"\n\x06linkId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12&\n\nfacilityId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12$\n\x08personId\x18\x03 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x0f\n\x07\x61\x63ttype\x18\x04 \x01(\t\"[\n\x0eLinkEnterEvent\x12\"\n\x06linkId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12%\n\tvehicleId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"[\n\x0eLinkLeaveEvent\x12\"\n\x06linkId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12%\n\tvehicleId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"o\n\x12PersonArrivalEvent\x12\"\n\x06linkId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x0f\n\x07legMode\x18\x02 \x01(\t\x12$\n\x08personId\x18\x03 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"q\n\x14PersonDepartureEvent\x12\"\n\x06linkId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x0f\n\x07legMode\x18\x02 \x01(\t\x12$\n\x08personId\x18\x03 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"g\n\x18PersonEntersVehicleEvent\x12$\n\x08personId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12%\n\tvehicleId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"g\n\x18PersonLeavesVehicleEvent\x12$\n\x08personId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12%\n\tvehicleId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"u\n\x10PersonMoneyEvent\x12$\n\x08personId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x0e\n\x06\x61mount\x18\x02 \x01(\x01\x12\x0f\n\x07purpose\x18\x03 \x01(\t\x12\x1a\n\x12transactionPartner\x18\x04 \x01(\t\"m\n\x10PersonStuckEvent\x12$\n\x08personId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\"\n\x06linkId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x0f\n\x07legMode\x18\x03 \x01(\t\"\xe7\x01\n\x18TransitDriverStartsEvent\x12$\n\x08\x64riverId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12%\n\tvehicleId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12*\n\x0etransitRouteId\x18\x03 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12)\n\rtransitLineId\x18\x04 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\'\n\x0b\x64\x65partureId\x18\x05 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"_\n\x12VehicleAbortsEvent\x12%\n\tvehicleId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\"\n\x06linkId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\"\xc1\x01\n\x19VehicleEntersTrafficEvent\x12$\n\x08\x64riverId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\"\n\x06linkId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12%\n\tvehicleId\x18\x03 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x13\n\x0bnetworkMode\x18\x04 \x01(\t\x12\x1e\n\x16relativePositionOnLink\x18\x05 \x01(\x01\"\xc1\x01\n\x19VehicleLeavesTrafficEvent\x12$\n\x08\x64riverId\x18\x01 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\"\n\x06linkId\x18\x02 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12%\n\tvehicleId\x18\x03 \x01(\x0b\x32\x12.matsim.pb.ProtoId\x12\x13\n\x0bnetworkMode\x18\x04 \x01(\t\x12\x1e\n\x16relativePositionOnLink\x18\x05 \x01(\x01\x42\'\n\x18org.matsim.core.utils.pbB\x0bProtoEventsb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'matsim.pb.Events_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\030org.matsim.core.utils.pbB\013ProtoEvents'
  _globals['_GENERICEVENT_ATTRSENTRY']._options = None
  _globals['_GENERICEVENT_ATTRSENTRY']._serialized_options = b'8\001'
  _globals['_EVENTBATCH']._serialized_start=81
  _globals['_EVENTBATCH']._serialized_end=127
  _globals['_EVENT']._serialized_start=130
  _globals['_EVENT']._serialized_end=1068
  _globals['_GENERICEVENT']._serialized_start=1070
  _globals['_GENERICEVENT']._serialized_end=1195
  _globals['_GENERICEVENT_ATTRSENTRY']._serialized_start=1151
  _globals['_GENERICEVENT_ATTRSENTRY']._serialized_end=1195
  _globals['_ACTIVITYENDEVENT']._serialized_start=1198
  _globals['_ACTIVITYENDEVENT']._serialized_end=1347
  _globals['_ACTIVITYSTARTEVENT']._serialized_start=1350
  _globals['_ACTIVITYSTARTEVENT']._serialized_end=1501
  _globals['_LINKENTEREVENT']._serialized_start=1503
  _globals['_LINKENTEREVENT']._serialized_end=1594
  _globals['_LINKLEAVEEVENT']._serialized_start=1596
  _globals['_LINKLEAVEEVENT']._serialized_end=1687
  _globals['_PERSONARRIVALEVENT']._serialized_start=1689
  _globals['_PERSONARRIVALEVENT']._serialized_end=1800
  _globals['_PERSONDEPARTUREEVENT']._serialized_start=1802
  _globals['_PERSONDEPARTUREEVENT']._serialized_end=1915
  _globals['_PERSONENTERSVEHICLEEVENT']._serialized_start=1917
  _globals['_PERSONENTERSVEHICLEEVENT']._serialized_end=2020
  _globals['_PERSONLEAVESVEHICLEEVENT']._serialized_start=2022
  _globals['_PERSONLEAVESVEHICLEEVENT']._serialized_end=2125
  _globals['_PERSONMONEYEVENT']._serialized_start=2127
  _globals['_PERSONMONEYEVENT']._serialized_end=2244
  _globals['_PERSONSTUCKEVENT']._serialized_start=2246
  _globals['_PERSONSTUCKEVENT']._serialized_end=2355
  _globals['_TRANSITDRIVERSTARTSEVENT']._serialized_start=2358
  _globals['_TRANSITDRIVERSTARTSEVENT']._serialized_end=2589
  _globals['_VEHICLEABORTSEVENT']._serialized_start=2591
  _globals['_VEHICLEABORTSEVENT']._serialized_end=2686
  _globals['_VEHICLEENTERSTRAFFICEVENT']._serialized_start=2689
  _globals['_VEHICLEENTERSTRAFFICEVENT']._serialized_end=2882
  _globals['_VEHICLELEAVESTRAFFICEVENT']._serialized_start=2885
  _globals['_VEHICLELEAVESTRAFFICEVENT']._serialized_end=3078
# @@protoc_insertion_point(module_scope)
